% Program for course "Topics in Econometric Models for Macroeconomics and
% Finance" by Carlo A. Favero, IGIER, Bocconi University. (02.2007)
%
% This program corresponds to Exercise Session 3 for questions 2).
% Essentially Affine term structure model with short rate as the only
% factor.
% This file is more general than what question 2) requires in that:
%   a)  it allows to estimate state dynamics with lags more than 1 (if set
%       lags > 1);
%   c)  rolling or recursive estimation can be made if set "wholesample=0".
%
% CONSTANT risk price. lam0 <> 0 and lam1 = 0.
% TIME-VARYING risk price: lam0, lam1 <> 0.
% Two step estimation: 1) OLS for state dynamics for mu, fai, omega;
%                      2) MLE for measurement equation to find out lam0,lam1.
%
% Data: Yields.xls, range: 1974.2 -2001.12, that corresponds to 2 - 336 row.
% Variables
% m01-y10: 11 yields
%
% Rolling estimation 120 periods from 1991:12 to 2001:11.
% Estimation window length 212; regression window 211.
%
%


% The structure of this file is the following:
% A. Data preparation and model selection
% B. Estimation (whole sample or  120 periods estimation)
%    In each period:
%    1. Define data
%    2. OLS estimation of unrestricted state dynamics VAR(1)
%    3. Define initial value of parameters used for MLE on other
%       parameters.
%    4. MLE, call likelihood function llfn_A012s.m
%    5. Control result, either repeat the estimation with another initial
%         value or accept the estimate and move forward.
% C. Save estimate for later analysis.
%
%


%% A. ***********  DATA PREPARATION AND MODEL SPECIFICATION  ***********
clear all;  % deletes global variables too
clc;

% Define path and load data
xdata = xlsread('Yields.xls',1, 'A2:L336'); % yield data (the second column is the 1m rate)

% Set global some parameters and intermediate result for global using or
% the purpose of checking.
global yy Xo XX Xfor H yn maturity K nlags;
global um aa0 bb0 AA1 BB1;
global mu fai sig del0 del1 lam1 crp;

% Define yield structure used in the estimation (11 maturities)
yname =  ['m01';'m02';'m03';'m06';'m09';'y01';'y02';'y03';'y05';'y07';'y10'];
maturity = [ 1;    2;    3;    6;    9;   12;   24;   36;   60;   84;  120]; 
colnumber = 2:1:12;  % corresponding column in the data set

% +++++++++++++++++++++    Define PARAMETERS ++++++++++++++++++++++++++
wholesample = 1;     % 1: estimate whole sample; 0: rolling estimation t=120
crp = 0; % 1: constant risk price; 0: time varying risk price
Roll=1; %Rolling estimation.
length = 335; % data length
yn = size(yname,1);  % number of yields
HN = [1 3 6 9 12 18 24];% forecasting horizon
nlags = 1;
K = nlags; % In compact form, there are seemingly K(>1) factors in the
% measurement equation when lags>1, we set factor
% loadings to lagged x(t) to be all zeros.
K2 = 0;    % Unobservable factors to be zero
lam1 = zeros(K,K);      % preset time varying risk prices to zeros for
% constant risk price case.

yield = xdata(1:length, colnumber);

%% B. ***************************  ESTIMATION  ***************************

if wholesample == 1;
    START = 1;
    LAST = length;
    FT = 1;
else
    START = 4;
    LAST = (START-1) + 212; % where the first rolling window ends
    T=length-LAST;  % left data periods for the first forecast
    FT = T;   % the forecast periods
end;


% ooooooooooooooooooooo MAIN LOOP: estimation    oooooooooooooooooooo
k=1;
while k <= FT;
    
    %:::: 1. DEFINE DATA to be used for yields and states
    yy = yield(START+(k-1)*Roll:LAST+k-1, :);  % it takes the whole sample
    yy = yy'./1200; % transform to annualised return per dollar.
    m1 = yy(1,:);
    nobs = size(m1,2);
    
    
    %:::: 2. OLS estimation of state dynamics VAR(p)
    X=m1(:,nlags:nobs);
    for i=1:nlags-1
        X=[X; m1(:,nlags-i:nobs-i)];
    end
    X=[ones(1,nobs-nlags+1); X]; %add the constant
    Xo = X;
    X=X'; m1=m1';
    X=X(1:nobs-nlags, :);
    Y=m1(nlags+1:nobs, :);
    gamav=(X'*X)\(X'*Y);    %OLS
    U=Y-X*gamav;
    sig = U'*U/(size(Y,1)-size(gamav,1));  %dof correction here
    gamav = [gamav'; zeros(nlags-1,1) eye(nlags-1) zeros(nlags-1,1)];  
    
    %:::: 2. OLS estimation of state dynamics VAR(1)
    %X = Xo(:,nlags:nobs);
    %Y = Xo(2:(K+1), nlags+H:nobs);
    %X = X(:, 1:nobs-nlags-H+1);
    %gamav = (Y*X')/(X*X');
    %omega = ((Y - gamav*X)*(Y-gamav*X)'/(nobs-H-nlags+1));
    %sigma = omega;
    
    %Transfer VAR(1) coef. of state equation to coef. of
    %  No-arbitrage model
    mu = gamav(:,1);
    fai = gamav(:,2:(1+K));
    del0 = 0; del1 = [1 zeros(1,K-1)]';  %the sole factor is the short rate
    
    %:::: 3. Set starting value for MLE.  (inizialize with risk neutrality)
    if k == 1;
        % At t = 1, suppose those macro factors are irrelevant.
        lam0_1 = 0;
        if crp == 1;
            bigthet = lam0_1;
        else
            lam1_11 = 0;
            bigthet = [lam0_1 lam1_11]';
        end;
        
    else
        bigthet = thetstar;
    end;
    
    %:::: 4. MLE (maximize likelihood estimation)
    warning off all;
    % set options.
    if k < 2;
        options = optimset('Display','iter','MaxIter',1e+005,'MaxFunEvals',1e+7,'TolFun',1e-7,'TolX',1e-10 );
    else
        options = optimset('MaxIter',0.2*(1e+005),'MaxFunEvals',1e+6,'TolFun',1e-7,'TolX',1e-10 );
    end;
    % likelihood maximisation, call likelihood function.
    % estimated parameter values:
    thetstar = fminsearch('llfn_A012s',bigthet,options);
    % correponding likelihood:
    fstar = llfn_A012s(thetstar)
    % collect results for each period:
    thetall(:,k)= thetstar;     % ML estimates of the price of risk parameters
    fstarall(k) = fstar;
    
    % one period ahead forecast
    mufai = [mu fai];
    Xhatk = mufai*Xfor;  Xhata(:,k) = Xhatk;
    yhatk=[AA1 BB1]*[1; Xhatk];
    yhata(:,k) = yhatk.*1200;
    
%     k
    %:::: 5. Control result, either repeat the estimation with
    %        another initial value or accept the estimate and
    %        move forward.
    if k>1
        fdiff = fstar-fstarall(k-1);
        if fdiff > 20 && ik <30 || fstar==-inf;
            k=k; ik=ik+1;
            if fstar==-inf;
                fik(:,ik)=NaN;
            else
                fik(:,ik)=fstar;
            end;
            thetik(:,:,ik)= thetstar;
            if ik==1;
                thetstar = zeros(size(thetstar,1),1);
            else
                thetstar = randn(size(thetstar,1),1);
            end;
        elseif fdiff > 20 && ik == 30 || fstar==-inf;
            if fstar==-inf;
                fik(:,ik)=NaN;
            else
                fik(:,ik)=fstar;
            end;
            thetik(:,:,ik)= thetstar;
            [fstar, mik] = min(fik);
            fstarall(k)=fstar;
            thetall(:,k)=thetik(:,:,mik);
            k=k+1; ik = 0;
        elseif fdiff<=20;
            k=k+1; ik = 0;
        end;
    elseif k == 1;
        k = k+1; ik = 0;
    end;

    %save intermediate result to file.
    fname = [cd,'\A01_2s_crp',num2str(crp),'_p',num2str(nlags)];
    save(fname, 'thetall', 'fstarall');
	
end; %% end k

%oooooooooooooooo  END MAIN LOOP: rolling estimation  ooooooooooooooo

%% C. ***********     SAVE ESTIMATE FOR LATER ANALYSIS      ************
fname = [cd,'\A01_2s_crp',num2str(crp),'_p',num2str(nlags)];
save(fname, 'thetall', 'fstarall');


%% D. ***********************     Forecast      ************************
if wholesample == 0;
    H = 1;
    ylag = yield(LAST:LAST+T-H,:)';
    y = yield(LAST+H:LAST+T,:)';
    
    serr_rw = (y-ylag).^2;
    serr = (y-yhata).^2;
    
    cumerr_rw = sum(serr_rw,2);
    cumerr = sum(serr,2);
    
    rmse_rw = sqrt(cumerr_rw/FT);
    rmse = sqrt(cumerr/FT);
    
    rwrRMSE = rmse./rmse_rw;
end;
