% Program for course "Topics in Econometric Models for Macroeconomics and
% Finance" by Carlo A. Favero, IGIER, Bocconi University. (02.2007)
%
% This program corresponds to Exercise Session 3 for questions 3).
% Essentially Affine term structure model with short rate as the only
% factor.
% This file is more general than what question 3) requires in that:
%   a)it allows to assume that the price measured without error is any yield
%     that we have in the data;  (AND NOT THE FIRST ONE)
%   c)rolling or recursive estimation can be made if set "wholesample=0".
%
% CONSTANT risk price. lam0 <> 0 and lam1 = 0.
% TIME-VARYING risk price: lam0, lam1 <> 0.
% Maximum likelihood estimation, Chen-Scott(1993).
%
% Data: Yields.xls, range: 1974.2 -2001.12, that corresponds to 2 - 336 row.
% Variables
% m01-y10: 11 yields
%
% Rolling estimation 120 periods from 1991:12 to 2001:11.
% Estimation window length 212; regression window 211.
%


% The structure of the file is the following:
% A. Data preparation and model selection
% B. Estimation (120 periods estimation)
%    In each period:
%    1. Define data
%    2. Define initial value of parameters used for MLE on other
%       parameters.
%    3. MLE, call likelihood function llfn_A01CS.m
%    4. Control result, either repeat the estimation with another initial
%         value or accept the estimate and move forward.
% C. Save estimate for later analysis.
% D. Forecast.
%

clear all;

%% A. ***********  DATA PREPARATION AND MODEL SPECIFICATION  ***********

% Define path and load data
xdata=xlsread('Yields.xls'); % yield data

% Set global some parameters and intermediate result for global using or
%   the purpose of checking.
global yy Xo Xu XX Xfor yn maturity K ne er;
global um ABne ABer aa0 bb0 AA1 BB1;
global mu fai sigma del0 del1 lam1 crp;

% Define yield structure used in the estimation
yname =  ['m01';'m02';'m03';'m06';'m09';'y01';'y02';'y03';'y05';'y07';'y10'];
maturity = [ 1;    2;    3;    6;    9;   12;   24;   36;   60;   84;  120];
colnumber = 2:1:12;

% +++++++++++++++++++++    Define PARAMETERS ++++++++++++++++++++++++++
wholesample = 1;     % 1: estimate whole sample; 0: rolling estimation t=120
crp = 0;    % 1: constant risk price; 0: time varying risk price
yn = size(yname,1);  % number of yields
Roll=1;       %Rolling estimation.
length = 335; % data length
K = 1;                  % number of factors
lam1 = zeros(K,K);      % preset time varying risk prices to zeros.
yield= xdata(1:length,2:12);
ne = 1;         %yields measured without error: m01
er = [colnumber(1:ne-1)-1 colnumber(ne+1:11)-1]; %yields measured with error

%% B. ***************************  ESTIMATION  ***************************
if wholesample == 1;
    START = 1;
    LAST = length;
    FT = 1;
else
    START = 4;
    LAST = (START-1) + 212; % where the first rolling window ends
    T = length - LAST;  % left data periods for the first forecast
    FT = T;             % the forecast periods
end;


% ooooooooooooooooooooo MAIN LOOP: estimation    oooooooooooooooooooo
k=1;
while k <= FT;
    
    %:::: 1. DEFINE DATA to be used for yields and states
    yy = yield(START+(k-1)*Roll:LAST+k-1, :);
    yy = yy'./1200; % transform to annualised return per dollar.
    mu = 0;
    del0 = mean(yy(1,:)); %short rate equation intercept.
    del1 = 1;
    
    %:::: 3. Set starting value for MLE.
    if k == 1;
        % At t = 1, suppose those macro factors are irrelevant.
        fai_11 = 0.99;
        sig_11 = (1-fai_11^2)*var(yy(1,:));
        lam0_1 = 0;
        if crp == 1;
            bigthet = [fai_11 sig_11 lam0_1];
        else
            lam1_1 = 0;
            bigthet = [fai_11 sig_11 lam0_1 lam1_1]';
        end;
        
    else
        bigthet = thetstar;
    end;
    
    %:::: 4. MLE (maximize likelihood estimation)
    warning off all;
    % set options.
    if k < 2;
        options = optimset('Display','iter','MaxIter',1e+005,'MaxFunEvals',1e+7,'TolX',1e-10 );
    else
        options = optimset('MaxIter',0.2*(1e+005),'MaxFunEvals',1e+6,'TolX',1e-10 );
    end;
    % likelihood maximisation, call likelihood function.
    % estimated parameter values:
    thetstar = fminsearch('llfn_A01CS',bigthet,options);
    % correponding likelihood:
    fstar = llfn_A01CS(thetstar)
    % collect results for each period:
    thetall(:,k)= thetstar;
    fstarall(k) = fstar;
    
    % one period ahead forecast
    mufai = [mu fai];
    Xhatk = mufai*Xfor;  Xhata(:,k) = Xhatk;
    yhatk=[AA1 BB1]*[1; Xhatk];
    yhata(:,k) = yhatk.*1200;
    
    k
    
    %:::: 5. Control result, either repeat the estimation with
    %        another initial value or accept the estimate and
    %        move forward.
    if k>1
        fdiff = fstar-fstarall(k-1);
        if fdiff > 20  && ik <30 || fstar==-inf;
            k=k; ik=ik+1;
            if fstar==-inf;
                fik(:,ik)=NaN;
            else
                fik(:,ik)=fstar;
            end;
            thetik(:,:,ik)= thetstar;
            if ik==1;
                thetstar = [thetstar(1:2,:) zeros(size(thetstar,1)-2,1)];
            else
                thetstar = [thetstar(1:2,:) randn(size(thetstar,1)-2,1)];
            end;
        elseif fdiff > 20 && ik == 30 || fstar==-inf;
            if fstar==-inf;
                fik(:,ik)=NaN;
            else
                fik(:,ik)=fstar;
            end;
            thetik(:,:,ik)= thetstar;
            [fstar, mik] = min(fik);
            fstarall(k)=fstar;
            thetall(:,k)=thetik(:,:,mik);
            k=k+1; ik = 0;
        elseif fdiff<=20;
            k=k+1; ik = 0;
        end;
    elseif k == 1;
        k = k+1; ik = 0;
    end;
	
	%save intermediate result to file.
	fname = [cd,'\A01_CS_crp',num2str(crp)];        
	save(fname, 'thetall', 'fstarall');
end; %% end k

%oooooooooooooooo  END MAIN LOOP: rolling estimation  ooooooooooooooo

%% C. ***********     SAVE ESTIMATE FOR LATER ANALYSIS      ************
fname = [cd,'\A01_CS_crp',num2str(crp)];
save(fname, 'thetall', 'fstarall');


%% D. ***********************     Forecast      ************************
if wholesample == 0;
    H = 1;
    ylag = yield(LAST:LAST+T-H,:)';
    y = yield(LAST+H:LAST+T,:)';
    
    serr_rw = (y-ylag).^2;
    serr = (y-yhata).^2;
    
    cumerr_rw = sum(serr_rw,2);
    cumerr = sum(serr,2);
    
    rmse_rw = sqrt(cumerr_rw/FT);
    rmse = sqrt(cumerr/FT);
    
    rwrRMSE = rmse./rmse_rw;
end;
