% Program for course "Topics in Econometric Models for Macroeconomics and
% Finance" by Carlo A. Favero, IGIER, Bocconi University. (02.2007)
%
% This program corresponds to Exercise Session 3 for questions 5)& 6)
% Essentially Affine term structure model with 3 independent factors.
% Set "wholesample=1" for question 5); set "wholesample=0" for 6).
%
% CONSTANT risk price. lam0 <> 0 and lam1 = 0.
% TIME-VARYING risk price: lam0, lam1 <> 0.
% Maximum likelihood estimation, Chen-Scott(1993).
%
% Data: Yields.xls, range: 1974.2 -2001.12, that corresponds to 2 - 336 row.
% Variables
% m01-y10: 11 yields
%
% Rolling estimation 120 periods from 1991:12 to 2001:11.
% Estimation window length 212; regression window 211.
%


% The structure of the file is the following:
% A. Data preparation and model selection
% B. Estimation (120 periods estimation)
%    In each period:
%    1. Define data
%    2. Define initial value of parameters used for MLE on other
%       parameters.
%    3. MLE, call likelihood function llfn_A01CS.m
%    4. Control result, either repeat the estimation with another initial
%         value or accept the estimate and move forward.
% C. Save estimate for later analysis.
% D. Forecast.
%
clc;
clear all;

%%          ***********************************
%               *       A. DATA PREPARATION       *
%               *               AND               *
%               *       MODEL SPECIFICATION       *
%               ***********************************

xdata=xlsread('Yields.xls'); % yield data

% Set global some parameters and intermediate result for global using or
%   the purpose of checking.
global yy Xo Xu XX Xfor yn maturity yne yer K K2;
global um ABne ABer aa0 bb0 AA1 BB1 VC ne er;
global mu fai sig lam1 del1 crp;

% Define yield structure used in the estimation
yname =  ['m01';'m02';'m03';'m06';'m09';'y01';'y02';'y03';'y05';'y07';'y10'];
maturity = [ 1;    2;    3;    6;    9;   12;   24;   36;   60;   84;  120];
colnumber = 2:1:12;

% +++++++++++++++++++++    Define PARAMETERS ++++++++++++++++++++++++++
wholesample = 1; %1: wholesample estimation
                 %0: rolling window 
yn = size(yname,1);  % number of yields
Roll=1;  %Rolling estimation.
length = 335; % data length
mnlags=1; % default 1. %maximal number of lags
crp = 0; % 1: constant risk price; 0: time varying risk price
K = 3;                 % number of factors
K2 = 3;                % number of mean-zero factor(s)
mu = zeros(K2,1);      % preset mean zero intercept of state VAR.
lam1 = zeros(K,K);     % preset time-varying risk prices to be zeros.
del1 = ones(K,1);      % preset factor loading on short rate to be ones.
tX = 8; % tolerance level of parameters precision from estimation


% for neorder = 0:8; % select the specification of yields measured w/o error
    neorder = 1;  % my choice of the maturities (see the exercise intructions) 
				  % if you want you can remove this line 
				  % and uncomment the for loop (line 71, line 262) in order to 
				  % select different maturities
				  % (the original code gives you 9 possibilities)
				
    if neorder == 0;
        %% ne0
        ne = [3 7 10];         %yields measured without error: m03 y02 y07
        er = [1 2 4 5 6 8 9 11]; %yields measured with error
        
    elseif neorder == 1;
        %% ne1
        ne = [3 7 11];         %yields measured without error: m03 y02 y10
        er = [1 2 4 5 6 8 9 10];
        
    elseif neorder == 2;
        %% ne2
        ne = [1 7 10];         %yields measured without error: m01 y02 y07
        er = [2 3 4 5 6 8 9 11];
        
    elseif neorder == 3;
        %% ne3
        ne = [1 7 11];         %yields measured without error: m01 y02 y10
        er = [2 3 4 5 6 8 9 10];
        
    elseif neorder == 4;
        %% ne4 (Moench (2006) selects this as A0(3) comparison. )
        ne = [1 6 11];         %yields messured without error: m01 y01 y10
        er = [2 3 4 5 7 8 9 10];
        
    elseif neorder == 5;
        %% ne5
        ne = [1 8 11];         %yields measured without error: m01 y03 y10
        er = [2 3 4 5 6 7 9 10];
        
    elseif neorder == 6;
        %% ne6
        ne = [3 8 11];         %yields measured without error: m03 y03 y10
        er = [1 2 4 5 6 7 9 10];
        
    elseif neorder == 7;
        %% ne7
        ne = [1 8 10];         %yields measured without error: m01 y03 y07
        er = [2 3 4 5 6 7 9 11];
        
    elseif neorder == 8;
        %% ne8
        ne = [3 8 10];         %yields measured without error: m03 y03 y07
        er = [1 2 4 5 6 7 9 11];
    end; %end if neorder
    
    yield= xdata(1:length,colnumber);  % Select yields from the data sheet.
    
    %%          ***********************************
    %               *                                 *
    %               *          B.  ESTIMATION         *
    %               *                                 *
    %               ***********************************
    
    if wholesample == 1;
        START = 1;
        LAST = length;
        FT = 1;
    else
        START = 4;
        LAST = (START-1) + 212; % where the first rolling window ends
        T = length - LAST;  % left data periods for the first forecast
        FT = T;             % the forecast periods
    end;
    
    
    % ooooooooooooooooooooo MAIN LOOP: estimation    oooooooooooooooooooo
    k=1;
    while k <= FT;
        
        %:::: 1. DEFINE DATA to be used for yields and states
        yy = yield(START+(k-1)*Roll:LAST+k-1, :);   %al primo giro l'ultima 
                                                    %ossevazione corrisponde 
                                                    %alla data 1991:12
                                                    %?giusta per predire 1992:1
                                                    
        yy = yy'./1200; % transform to annualised return per dollar.
        nlags = mnlags;
        del0 = mean(yy(1,:));   %again, the factors are zero-mean so we have 
                                %to set delta0 to the mean of the short
                                %rate 
        
        %:::: 2. Set starting value for MLE.
        if k == 1;
            % fai lower triangular
            fai_11 = 0.99;
            fai_22 = 0.9;
            fai_33 = 0.8;
            % sig diagonal  (OMEGA!!)
            sig_11 = var(yy(1,:));
            sig_22 = 0.04*var(yy(1,:));
            sig_33 = 0.02*var(yy(1,:));
            % constant risk price, start from risk neutral case
            lam0_1 = 0;
            lam0_2 = 0;
            lam0_3 = 0;
            % time-varying risk price, start from risk neutral case
            lam1_11 = 0;
            lam1_22 = 0;
            lam1_33 = 0;
            % Vectorise all parameter for MLE input
            if crp == 1;
                bigthet = [  fai_11 fai_22 fai_33...
                    sig_11 sig_22 sig_33...
                    lam0_1 lam0_2 lam0_3]';
            else
                bigthet = [  fai_11 fai_22 fai_33...
                    sig_11 sig_22 sig_33...
                    lam0_1 lam0_2 lam0_3...
                    lam1_11 lam1_22 lam1_33]';
            end
            
        else
            % recieve previous estimate as initial value
            bigthet = thetstar;
        end;
        
        
        %:::: 3. MLE (maximize likelihood estimation)
        warning off all;
        % set options.
        if k < 2;
            options = optimset('Display','iter','MaxIter',1e+005,'MaxFunEvals',1e+6,'TolFun',10^(-tX+1),'TolX',10^(-tX)  );
        else
            options = optimset('MaxIter',0.2*(1e+005),'MaxFunEvals',1e+6,'TolFun',10^(-tX+1),'TolX',10^(-tX) );
        end;
        % likelihood maximisation, call likelihood function.
        % estimated parameter values:
        thetstar = fminsearch('llfn_A03iCS',bigthet,options);
        % correponding likelihood:
        fstar = llfn_A03iCS(thetstar)
        % collect results for each period:
        thetall(:,k)= thetstar;
        fstarall(k) = fstar;
        
        % one period ahead forecast  
        % Xfor ?l'ultima osservazione di Xu della finestra (Xfor(2:end)-Xu(:,end))
        mufai = [mu fai];
        Xhatk = mufai*Xfor;  Xhata(:,k) = Xhatk;
        yhatk=[AA1 BB1]*[1; Xhatk];
        yhata(:,k) = yhatk.*1200; %gross rate annual
       
        % RW forecst simply take the last available observation at time t
		% the next 4 lines are redundant but ease RMSFE computation
        yhat_rw(:,k) = yield(LAST+k-1,:)';
        if wholesample == 0
            ytrue(:,k) = yield(LAST+k,:)';
        end
        
        k
        
        %:::: 4. Control result, either repeat the estimation with
        %        another initial value or accept the estimate and
        %        move forward.
        if k>1
            fdiff = fstar-fstarall(k-1);
            if fdiff > 150;
                k=k;
                thetstar = [thetall(1:9,k-1); zeros(size(thetstar,1)-9,1)];
            else
                k=k+1;
            end;
        else
            k = k+1;
        end;
        
        %save intermediate result to file.
        fname = [cd,'\A03i_ne',num2str(neorder),'_crp',num2str(crp),'k',num2str(FT)];
        save(fname, 'thetall', 'fstarall');
        
    end; %% end "while" for k
    
    %oooooooooooooooo  END MAIN LOOP: rolling estimation  ooooooooooooooo
    
%%              ***********************************
    %               *        C. SAVE ESTIMATE         *
    %               *               FOR               *
    %               *         LATER ANALYSIS          *
    %               ***********************************
    fname = [cd,'\A03i_ne',num2str(neorder),'_crp',num2str(crp),'k',num2str(FT)];
    save(fname, 'thetall', 'fstarall');
    if wholesample ==0
    if k>1
        fname = [cd,'\A03i_ne',num2str(neorder),'_crp',num2str(crp),'k',num2str(FT)];
        save(fname, 'thetall', 'fstarall','ytrue','yhata','yhat_rw');
    end
    end
% end; % neorder

% I saved in the file A03i_ne1_crp0k120_RMSFE the forecasting results
% I then recall the file to print out the results
% save('F:\Documenti - ale.gobbi\prog\matlab\favero\es3\A03i_ne1_crp0k120_RMSFE', 'thetall', 'fstarall','ytrue','yhata','yhat_rw', 'maturity', 'FT');

%% RMSFE
if wholesample == 0
    load A03i_ne1_crp0k120_RMSFE
    plot(1:FT,[ytrue(1,:);yhata(1,:);yhat_rw(1,:)])
    legend('1m true','1m hat affine','1n hat rw')
    RMSFE_affine = sqrt(mean((ytrue-yhata).^2,2));
    RMSFE_rw = sqrt(mean((ytrue-yhat_rw).^2,2));
    ratio_RMSFE_arw = RMSFE_affine./RMSFE_rw;
    
    fprintf('***************************************************************** \n')
    fprintf('***********************     RMSFE      ************************** \n')
    fprintf('***************************************************************** \n')
    fprintf('  maturity (months)   RMSFE affine   RMSFE RW   ratio  \n')
    fprintf('           %6i     %9.2f  %9.2f  %9.2f     \n', ...
        [maturity,RMSFE_affine,RMSFE_rw, ratio_RMSFE_arw]' )
    fprintf('***************************************************************** \n')
    fprintf('***************************************************************** \n')
end

%% my plot for question 5
if wholesample == 1
	% I create the calendar in MATLAB form
    y_init = 1974; m_init = 2;
    y_fin   = 2001; m_fin   = 12;
    cal_m_vec_y = kron((y_init:y_fin)',ones(12,1));
    cal_m_vec_m = kron(ones(y_fin-y_init+1,1),(1:12)');
    index = ones(size(cal_m_vec_y,1),1);
    index(1:m_init-1) = 0;
    index(end-(12 - m_fin)+1:end) = 0;
    cal_m = datenum(cat(2,cal_m_vec_y(index==1),...
        cal_m_vec_m(index==1),ones(sum(index==1),1)) );
    
    plot(cal_m,Xu)
    legend('Xu 3m','Xu 2y','Xu 10y')
    grid on;
    % line([1 length],[0 0], 'Color', [.7 .7 .7],'LineStyle' , ':') % it conflicts with datetick so I delete it
    datetick('x');     Xlim([cal_m(1), cal_m(end)])
    title('Latent Factors')
end
Xlim([cal_m(1), cal_m(end)])
%% April 30th 2011

%% 1. compare the laten factors with Nelson-Siegel factors
if wholesample == 1
    lamda=0.077;         % as in Diebold, Rudebusch and Aruoba(2003)
    Roll=1;              % Rolling estimation (1) or recursive estimation (0).
    mat = 1:1:120;
    pNS1 = ones(1,120);
    pNS2 = zeros(1,120);
    pNS3 = zeros(1,120);
    for imat = 1:120;
        pNS2(imat) = (1 - exp(-lamda*imat))/(lamda*imat);
        pNS3(imat) = pNS2(imat) - exp(-lamda*imat);
    end;

    CCw = [pNS1; pNS2; pNS3]'; %Coefficient matrix for all maturity 1:120
    CC = CCw(maturity',:);
    yield= xdata(1:length,2:12);
    NS = ((CC'*CC)\eye(size(CC,2)))*CC'*yield';

    tm = 1974+2/12:1/12:2001+12/12;
    tm = tm';
    figure(2)
    subplot(3,1,1)

    plot(tm,NS(1,:),'r-', tm, Xu(2,:)*1e3+mean(yield(:,1)), 'b:');
    %      plot(tm,NS(1,:),'r-');
    h = legend('NS 1st factor','Xu 2y',1);
    %xlabel('Time')
    ylabel('%')
    %title('');

    subplot(3,1,2)
    plot(tm,NS(2,:),'r-', tm,  Xu(1,:)*1e3, 'b:');
    %      plot(tm,NS(2,:),'r-');
    h = legend('NS 2nd factor','Xu 3m',1);  
    %xlabel('Time')
    ylabel('%')
    %title('');

    subplot(3,1,3)
    plot(tm,NS(3,:),'r-', tm,  -Xu(3,:)*1e3, 'b:');
    %      plot(tm,NS(3,:),'r-');
    h = legend('NS 3rd factor','Xu 10y',1);
    xlabel('Time')
    ylabel('%')
    %title('');
end
%% 2. long term forecasting
% in this case we do the long term (one-step ahead forecasts by using the initial value at 1991:12 
% and estimator for window 1975:02-1991:12). Both affine model and random-walk perform really bad.
if wholesample==1
    load A03i_ne1_crp0k120_RMSFE.mat
    theta = thetall;
    phi = diag(theta(1:K));
    omega = diag(theta(K+1:2*K));
    lam0 = theta(2*K+1:3*K)';
    lam1 = diag(theta(3*K+1:4*K));
    del0 = mean(yy(1,:));

    AA0 = zeros(1, 120);
    BB0 = zeros(K, 120);
    AA0(1) = - del0;      aa0 = -AA0;
    BB0(:,1) = - del1;    bb0 = -BB0;
    for i = 2 : 120;
        AA0(i) = AA0(i-1) + BB0(:,i-1)'*(mu - omega*lam0) + 0.5*BB0(:,i-1)'*omega*BB0(:,i-1) - del0;
        BB0(:,i) = (phi' - lam1'*omega')*BB0(:,i-1) - del1;
        aa0(i) = - AA0(i)./i;
        bb0(:,i) = - BB0(:,i)./i;
    end;
    AA1 = aa0(maturity')';
    BB1 = bb0(:, maturity')';

    Xs = zeros(K,121);
    Xs(:,1)= Xu(:,end-120);
    for i=2:120
        Xs(:,i)=phi*Xs(:,i-1);  %mu=[0;0;0]
    end
    Xss = Xs(:,2:end);

    for j=1:120
        ys(:,j) = 1200*(AA1 + BB1*Xss(:,j));  %gross rate annual
    end
    for j=1:120
        yrw(:,j) = yield(end-120,:)';
    end
    figure
    plot(1:FT,[ytrue(1,:);ys(1,:);yrw(1,:)])
    title('long term forecasts')
    legend('1m true','1m hat affine','1n hat rw')

end