% Program for course "Topics in Econometric Models for Macroeconomics and 
% Finance" by Carlo A. Favero, IGIER, Bocconi University. (02.2007)
%
% This program corresponds to Exercise Session 3 for questions 1).
% Pricing bonds with higher maturities for representative agents with
% essentially affine term structure model.
%
% This exercise aims to practice bond pricing given:
%    1) State dynamics;       X(t) = mu + fai*X(t-1)) + v(t), 
%                                    omega = cov(v(t))
%    2) Short rate equation:  r(t)= del0 + del1*X(t)
%    3) Prices of risk: (representative agent's risk aversion behavior)
%                             lam(t) = lam0 + lam1*X(t)    
%
% The above three equations contains all fundamental parameters that one
% needs to price higher maturity bonds. 
% Namely, vector of parameters: {mu,fai,omega,del0,del1,lam0,lam1}
%
%
% Denote log bond price of maturity n at time t as p(t,n)
% p(t,n) is affine (linear) function of state X(t), so is yield with
% maturity n at time t.
% 
% y(t,n) = -(1/n)*p(t,n);
% p(t,n) = A(n) + B(n)* X(t);
% y(t,n) = a(n) + b(n)* X(t); 
%          Hence, a(n) = -(1/n)*A(n); b(n) = -(1/n)*B(n)
%
% 
% The existence of a stochastic discount factor (SDF) of the form:
%          m(t+1) = exp[-r(t)-0.5*lam(t)'*omega*lam(t)-lam(t)'*v(t+1)];
% leads to no-arbitrage restrictions on A(n), B(n), so that {A(n),B(n)}
% follows a partial differential equations and functions of the basic
% parameters. 
%
% A(n+1) = A(n) + B(n)' * (mu - omega*lam0) + 0.5*B(n)'*omega*B(n) - lam0;
% B(n+1)' = B(n)'*(fai - omega*lam1) - del1';
%           with A(1) = -del0; B(1) = - del1;
%
%
% Exercise 1: given parameters, calculate {A(n),B(n)}, {a(n),b(n)} for
% different cases of a one-factor model, where the one factor is the short
% rate (1month)
%
% a) Case of states: 
% CState: 1. Random Walk; 
%         2. AR(1); 
%         3. White Noise.
% b) Case of representative agents: 
% CAgent: 1. Risk neutral;
%         2. Constant risk price;
%         3. Time-varying risk price.
% c) Case of short rate equation:
% Since we assume the short rate is the only factor, r(t)=X(t)
%         del0 = 0; del1 = 1.
clear all;

%% A. ***********   Choose different scenarios (parameters)   *********** 
K = 1; % number of factor(s)
close all
for CState = 1:3;   %CState is a choice variable switching the state dynamics
    
    for CAgent = 1:3;   %CAgent is a choice variable switching the risk prices
        
        if CState == 1
            state_string = 'Random Walk';
            mu = 0; fai = 1; omega = 0.0006^2;
        elseif CState == 2
            state_string = 'AR(1)';
            mu = 0.0001; fai = 0.98; omega = 0.0006^2;
            mean_unc = mu/(1-fai); %unconditional mean
        elseif CState == 3
            state_string = 'White Noise';
            mu = 0.001; fai = 0; omega = 0.003^2;
        end
        
        if CAgent == 1;
            agent_string = ' Risk Neutral agents';
            lam0 = 0; lam1 = 0;
        elseif CAgent == 2;
            agent_string = ' Constant Risk Price';
            lam0 = -800; lam1 = 0;
        elseif CAgent == 3;
            agent_string = ' Time-varying Risk Price';
            lam0 = 500; lam1 = -10^5;
        end;
        
        % Short rate dynamics
        del0 = 0;
        del1 = 1;
        
        
        % B. Calculate the PDE of coefficients for yields with all maturity on
        %     the measurement equations;
        
        % Factor loading matrix AA, BB for yields up to 120 months
        AA0 = zeros(1, 120);
        BB0 = zeros(K, 120);
        AA0(1)   = - del0;    aa0 = -AA0;
        BB0(:,1) = - del1;    bb0 = -BB0;
        for i = 2 : 120;
            AA0(i)   = AA0(i-1) + BB0(:,i-1)'*(mu - omega*lam0) + 0.5*BB0(:,i-1)'*omega*BB0(:,i-1) - del0;
            BB0(:,i) = (fai' - lam1'*omega')*BB0(:,i-1) - del1;
            aa0(i)   = - AA0(i)./i;
            bb0(:,i) = - BB0(:,i)./i;
        end;
        
        % plot of AA0, BB0, aa0, bb0
        figure('Name', strcat(state_string,' with',agent_string))
        subplot(2,2,1)
        plot([1:120],AA0); xlim([0,120]); set(gca,'XTick',[1,12, 24, 60, 120])
        xlabel('maturity');  ylabel('A');
        subplot(2,2,2)
        plot([1:120],BB0); xlim([0,120]); set(gca,'XTick',[1,12, 24, 60, 120])
        xlabel('maturity');  ylabel('B');
        subplot(2,2,3)
        plot([1:120],aa0); xlim([0,120]); set(gca,'XTick',[1,12, 24, 60, 120])
        xlabel('maturity');  ylabel('a');
        subplot(2,2,4)
        plot([1:120],bb0); xlim([0,120]); set(gca,'XTick',[1,12, 24, 60, 120])
        xlabel('maturity');  ylabel('b');
        
        % C. Price bonds with given states x(t)
        x1 = 0.0025;
        x2 = 0.009;
        x3 = 0.0081518;
%         x = x1;  % switch manually among state x1, x2 and x3
%         x = x2;  % switch manually among state x1, x2 and x3 
        x = x3; 
        pp = [AA0' BB0']*[1; x]; % Bond prices in log
        PP = exp(pp);           % Bond prices for face value = 1
        yy(:,CAgent,CState) = [aa0' bb0']*[1; x]*1200; % Bond yields (continuously compounded)
                                                % annualized      
    end
end

%% yield curve plot
figure('Name', strcat('yield curve for x = ',num2str(x)))
subplot(3,3,1)
plot(yy(:,1,1))
axis([0,121,min(yy(:,1,1))-0.0005,max(yy(:,1,1))+0.0005]);
title('Risk Neutral')
%xlabel('Maturity')
ylabel('Random Walk')
subplot(3,3,2)
plot(yy(:,2,1))
axis([0,121,min(yy(:,2,1))-0.0005,max(yy(:,2,1))+0.0005]);
title('Constant Risk Price')
%xlabel('Maturity')
%ylabel('Yields')
subplot(3,3,3)
plot(yy(:,3,1))
axis([0,121,min(yy(:,3,1))-0.0005,max(yy(:,3,1))+0.0005]);
title('Time-varying Risk Price')
%xlabel('Maturity')
%ylabel('Yields')

subplot(3,3,4)
plot(yy(:,1,2))
axis([0,121,min(yy(:,1,2))-0.0005,max(yy(:,1,2))+0.0005]);
%title('Risk Neutral')
%xlabel('Maturity')
ylabel('AR(1)')
subplot(3,3,5)
plot(yy(:,2,2))
axis([0,121,min(yy(:,2,2))-0.0005,max(yy(:,2,2))+0.0005]);
%title('Constant Risk Price')
%xlabel('Maturity')
%ylabel('Yields')
subplot(3,3,6)
plot(yy(:,3,2))
axis([0,121,min(yy(:,3,2))-0.0005,max(yy(:,3,2))+0.0005]);
%title('Time-varying Risk Price')
%xlabel('Maturity')
%ylabel('Yields')

subplot(3,3,7)
plot(yy(:,1,3))
axis([0,121,min(yy(:,1,3))-0.0005,max(yy(:,1,3))+0.0005]);
%title('Risk Neutral')
xlabel('Maturity')
ylabel('White Noise')
subplot(3,3,8)
plot(yy(:,2,3))
axis([0,121,min(yy(:,2,3))-0.0005,max(yy(:,2,3))+0.0005]);
%title('Constant Risk Price')
xlabel('Maturity')
%ylabel('Yields')
subplot(3,3,9)
plot(yy(:,3,3))
axis([0,121,min(yy(:,3,3))-0.0005,max(yy(:,3,3))+0.0005]);
%title('Time-varying Risk Price')
xlabel('Maturity')
%ylabel('Yields')