% Program for paper "Term Structure Forecast: No-arbitrage Restrictions vs.
%   Large Information set" (2007) by Carlo A. Favero, Linlin Niu and Luca
%   Sala, IGIER, Bocconi University.
%
% No-arbitrage model of term structure with 1 latent factor and 2 observa-
% ble macro variable (unemployment rate, CPI inflation).
% With constant risk price. lam0 <> 0 and lam1 = 0.
% By maximum likelihood estimation (Chen-Scott (1993) method).
%
% This file estimate a mixed model with constant risk price.
% Rolling estimation 120 periods from 1991:12 to 2001:11.
% Estimation window length 211; regression window 210.
% Iterated forecast.
% Forecast comparison 74 periods: 1995:11 to 2001:12.
% Forecast comparison horizons: 1 month till 24 months ahead.
% Some forecast before 1995:11 are not utilised, because we take these
%   periodss as training for the program to find optimal estimate, as
%   we input the estimate of period (t) to be the initial condition for
%   MLE procedure of period (t+1).
%
% Variables
% m01-y10: 11 yields
% DATA: data.xls, range: 1974.2-2001.12, that corresponds to 2 - 336 row #.
%
% The structure of the file is the following:
% A. Data preparation and model selection
% B. Estimation (120 periods estimation)
%    In each period:
%    1. Define data
%    2. Define initial value of parameters used for MLE
%    3. MLE, call likelihood function llfn_mix.m
%    4. MLE standard error, call likelihood functtion llfn_mix_sd.m
%    4. Control result, either repeat the estimation with another initial
%         value or accept the estimate and move forward.
% C. Save estimate for later analysis.
%
% Model specification:
% 1) Parameters.
%    Three latent mean-zero state variables. VAR(1) coefficient "fai" lower
%    triangular; variance covariance matrix "sigma/sig" of VAR innovation
%    diagonal; Constant of short rate equation "del0" is set to the mean
%    of short rate minus the contributions from observable factors;
%    Coefficients of short rate on latent variables are set
%    to be a vector of ones for normalisation.
% 2) Measurement error.
%    According to Chen-Scott (1993) MLE method, three (same as # of latent
%    variables) yields are assumed to be measured without error. We propose
%    here different specifications of this triple (1 short, 1 medium, 1
%    long) of yields in the selection vector "ne" and "neorder". We show
%    here the estimation of "ne1" with "neorder"(3month 2year 10year).


clear all;
%% A. ***********  DATA PREPARATION AND MODEL SPECIFICATION  ***********

% Define path and load data
xdata1=xlsread('Yields.xls'); % yield data
xdata2=xlsread('TRmacro.xls');

% Set global some parameters and intermediate result for global using or
%   the purpose of checking.
global yy Xo Xu XX Xfor yn yne yer maturity K K2 meanX;
global um ABne ABer aa0 bb0 AA1 BB1 VC ne er;
global mu fai sig lam1 crp;

% Define yield structure used in the estimation
yname =  ['m01';'m02';'m03';'m06';'m09';'y01';'y02';'y03';'y05';'y07';'y10'];
maturity = [ 1;    2;    3;    6;    9;   12;   24;   36;   60;   84;  120];
colnumber = 2:1:12;

%yname =  ['m01';'m02';'m03';'m06';'m09';'y01';'y02';'y03';'y05'];
%maturity = [ 1;    2;    3;    6;    9;   12;   24;   36;   60];
%colnumber = [9;   21;   22;   23;   24;   19;   17;   20;   16];

% +++++++++++++++++++++    Define PARAMETERS ++++++++++++++++++++++++++
wholesample = 0;     % 1: estimate whole sample; 0: rolling estimation t=120
yn = size(yname,1);  % number of yields
Roll=1;
length = 335; % data length
mnlags=1; % default 12. %maximal number of lags
mnfac=2; %number of macro factors
crp = 1; % 1: constant risk price; 0: time varying risk price
HN = [1 3 6 9 12 18 24];%% %forecasting horizon
K = 1+mnfac;                 % number of factors
K2 = 1;                % number of unobservable factor(s)
ne = 7;                % the yield measured without error: 2 year
er = [colnumber(1:ne-1)-1 colnumber(ne+1:11)-1]'; %yields measured with error
yield= xdata1(1:length,colnumber);

%% B. ***************************  ESTIMATION  ***************************
if wholesample == 1;
    START = 1;
    LAST = length;
    FT = 1;
else
    START = 4;
    LAST = (START-1) + 212; % where the first rolling window ends
    T = length - LAST;  % left data periods for the first forecast
    FT = T;             % the forecast periods
end;

% ooooooooooooooooooooo MAIN LOOP: estimation    oooooooooooooooooooo
k=1;
while k <= FT;
    
    %:::: 1. DEFINE DATA to be used for yields and states
    yy = yield(START+(k-1)*Roll:LAST+k-1, :)';
    yy = yy./1200;
    un= xdata2(START+(k-1)*Roll:LAST+k-1,1)./1200;   %Unemployment
    infl = xdata2(START+(k-1)*Roll:LAST+k-1,2)./1200;
    ma = [un infl]';
    Xo = ma;
    
    %:::: 2. Set starting value for MLE.
    lam1 = zeros(K,K);
    meanX = [mean(Xo,2); zeros(K2,1)];
    nlags = 1;
    nobs = size(Xo,2);
    if k == 1;
        %fai lower triangular
        fai_11 = 0;
        fai_21 = 0;
        fai_22 = 0;
        fai_31 = 0;
        fai_32 = 0;
        fai_33 = 0;
        %sig diagonal     (OMEGA)
        sig_11 = var(Xo(1, :)); %(1-fai_11^2)*var(Xo(1, :));
        sig_22 = var(Xo(2, :)); %(1-fai_22^2)*var(Xo(2, :));
        sig_33 = var(yy(1,:));
        % factor loadings for observable factors
        del1_1 = 0;
        del1_2 = 0;
        % constant risk price, start from risk neutral and A0(3)
        lam0_1 = 0;
        lam0_2 = 0;
        lam0_3 = 0;
        % Vectorise all parameter for MLE input
        bigthet = [  fai_11...
            fai_21 fai_22...
            fai_31 fai_32 fai_33...
            sig_11 sig_22 sig_33...
            del1_1 del1_2...
            lam0_1 lam0_2 lam0_3]';
    else
        % recieve previous estimate as initial value
        bigthet = thetstar;
    end;
    
    
    %:::: 4. MLE (maximize likelihood estimation)
    warning off all;
    % set options.
    if k < 2;
        options = optimset('Display','iter','MaxIter',1e+005,'MaxFunEvals',1e+7,'TolX',1e-10 );
    else
        options = optimset('MaxIter',0.2*(1e+005),'MaxFunEvals',1e+6,'TolX',1e-10 );
    end;
    % likelihood maximisation, call likelihood function.
    % estimated parameter values:
    thetstar = fminsearch('llfn_mix',bigthet,options);
    fstar = llfn_mix(thetstar)
    thetall(:,k)= thetstar;
    fstarall(k) = fstar;
    
    % one period ahead forecast
    mufai = [mu fai];
    Xhatk = mufai*Xfor;  Xhata(:,k) = Xhatk;
    yhatk=[AA1 BB1]*[1; Xhatk];
    yhata(:,k) = yhatk.*1200;
    
    
    k
    %:::: 5. Control result, either repeat the estimation with
    %        another initial value or accept the estimate and
    %        move forward.
    if k>2
        fdiff = fstar-fstarall(k-1);
        if fdiff > 150;
            k=k;
            thetstar = [thetall(1:12,k-1,ih); zeros(size(thetstar,1)-12,1)];
        else
            k=k+1;
        end;
    else
        k = k+1;
    end;
    
    fname = [cd,'\cs1_2ma_var_lam01_k',num2str(FT)];
    save(fname, 'thetall', 'fstarall');
end; %% end k
%oooooooooooooooo  END MAIN LOOP: rolling estimation  ooooooooooooooo

%% C. ***********     SAVE ESTIMATE FOR LATER ANALYSIS      ************
fname = [cd,'\cs1_2ma_var_lam01_k',num2str(FT)];
save(fname, 'thetall', 'fstarall');

%%          ***********************************
%               *        D. Standard Error        *
%               *                                 *
%               *             Estimation          *
%               ***********************************
% add by Haoxi Yang (April, 2011)
if wholesample == 1   % this is for whole sample estimtors you can also change it for every window in rolling case
    global sig
    fname = [cd,'\cs1_2ma_var_lam01_k',num2str(FT)];
    load(fname);
    theta = thetall(:,:);
    sig = eye(K)*10^(-6);
    sig(1,1) = theta(K^2+1);
    sig(2,2) = theta(K^2+2);
    sig(3,3) = theta(K^2+3);
    theta = [theta(1:K^2);theta(K^2+K+1:end)];
    t = length-1;
    [H VCV std] = Robust_VCV(@llfn_mix_sd,theta,t);
    %save intermediate result to file.
    fname = [cd, '\std_cs1_2ma_var_lam01_k',num2str(FT)];
    save(fname, 'H', 'VCV','std');
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%