% Standard Error Estimation Funcetion
% Haoxi Yang Nov. 25,2010

function [H VCV std] = Robust_VCV(f,theta,t)
% function [VCV H scores gross_scores] = Robust_VCV(f,theta,nw,T)
% PURPOSE: 
%      Computes Robust Variance-Covariance Matrix for MLE Estimators
%
% USAGE:  
%      [VCV scores hess gross_scores] = Robust_VCV(f,theta,nw,varargin)
%
% INPUTS:
%      f                - function name, likelihood function (such as f=@llfn_tays1_rest_sd) 
%      theta            - vector of parameters (n x 1)
%      nw               - newey-west matrix (set it equal to 0 here)
%      t                - window size of estimation(i.e mRoll)
% OUTPUTS:
%      VCV              - robust estimated variance-covariance matrix
%      H                - estimation of Hessian matrix
%      scores           - individual scores value in every period: T*n matrix(T:time period and n: number of parameters) 
%      gross_scores     - sum of the individual scores value T*1 matrix
%
% COMMENTS:
%      Code for Hessian matrix originally from COMPECON toolbox [www4.ncsu.edu/~pfackler]
%      documentation modified to fit the format of the Ecoometrics Toolbox
%      by James P. LeSage, Dept of Economics
%      University of Toledo
%      2801 W. Bancroft St,
%      Toledo, OH 43606
%      jlesage@spatial-econometrics.com
%      Further modified (to do 2-sided numerical derivs, rather than 1) by:
%      Author: Kevin Sheppard
%      kevin.sheppard@economics.ox.ac.uk
%      Revision: 3    Date: 4/1/2004

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Initial input setting
n = size(theta,1);
h=max(abs(theta/2),1e-2)*eps^(1/3);
h = diag(h);
% [llfn ll] = feval(f,theta,varargin{:});
% t = 104; %% time peiods
llfnp = zeros(n,1);
llfnm = zeros(n,1);
llp = zeros(t,n);
llm = zeros(t,n);
%% calculate individual likelihood and gross likelihood values
for i=1:n
   thetap = theta + h(:,i);
   [llfnp(i),llp(:,i)] = feval(f,thetap); 
%    [llfnp(i),llp(:,i)] = feval(f,thetap,varargin{:}); 
   thetam = theta - h(:,i);
   [llfnm(i),llm(:,i)] = feval(f,thetam);
%    [llfnm(i),llm(:,i)] = feval(f,thetam,varargin{:}); 
end
scores = zeros(t,n);
gross_scores = zeros(n,1);
h = diag(h);
for i=1:n
   scores(:,i) = (llp(:,i)-llm(:,i))./(2*h(i));    
   gross_scores(:,i) = (llfnp(i)-llfnm(i))./(2*h(i));
end 
%% Hessian matrix and Robust variance covariance matrix estimation
H = hessian_2sided(f,theta);
A = H/t; 
%A = H;  %?????????????????
Ainv = A^(-1);
B = cov(scores);
VCV = (Ainv*B*Ainv);
VCV_1= VCV./t;
std = sqrt(diag(VCV_1));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% SECTION: HESSIAN MATRIX ESTIMATION %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function H = hessian_2sided(f,x)
% function H = hessian_2sided(f,x,varargin)
% PURPOSE: 
%      Computes 2-sided finite difference Hessian
%
% USAGE:  
%      H = hessian_2sided(func,x,varargin)
%
% INPUTS:
%      func         - function name, fval = func(x,varargin)
%      x            - vector of parameters (n x 1)
%      varargin     - optional arguments passed to the function
%
% OUTPUTS:
%      H            - finite differnce, 2-sided hessian
try
    feval(f,x);
%     feval(f,x,varargin{:});
catch
    error('There was an error evaluating the function.  Please check the arguements.');
end


n = size(x,1);
fx = feval(f,x);
% fx = feval(f,x,varargin{:});

% Compute the stepsize (h)
h = eps.^(1/3)*max(abs(x),1e-2);
xh = x+h;
h = xh-x;    
ee = sparse(1:n,1:n,h,n,n);

% Compute forward and backward steps
gp = zeros(n,1);
for i=1:n
    gp(i) = feval(f,x+ee(:,i));
%     gp(i) = feval(f,x+ee(:,i),varargin{:});
end
gm = zeros(n,1);
for i=1:n
    gm(i) = feval(f,x-ee(:,i));
%     gm(i) = feval(f,x-ee(:,i),varargin{:});
end


H=h*h';
Hm=H;
Hp=H;
% Compute "double" forward and backward steps
for i=1:n
    for j=i:n
        Hp(i,j) = feval(f,x+ee(:,i)+ee(:,j));
%        Hp(i,j) = feval(f,x+ee(:,i)+ee(:,j),varargin{:});
        Hp(j,i)=Hp(i,j);
        Hm(i,j) = feval(f,x-ee(:,i)-ee(:,j));
%         Hm(i,j) = feval(f,x-ee(:,i)-ee(:,j),varargin{:});
        Hm(j,i)=Hm(i,j);        
    end
end

%Compute the hessian
for i=1:n
    for j=i:n
        H(i,j) = (Hp(i,j)-gp(i)-gp(j)+fx+fx-gm(i)-gm(j)+Hm(i,j))/H(i,j)/2;
        H(j,i) = H(i,j);
    end
end
