% Program for paper "Term Structure Forecast: No-arbitrage Restrictions vs.
%   Large Information set" (2007) by Carlo A. Favero, Linlin Niu and Luca 
%   Sala, IGIER, Bocconi University.
%
% No-arbitrage model of term structure with Taylor Rule state factors. 
% Traditional Taylor Rule variables: FFR, CPIinfl, Unempl. 
% CONSTANT risk price. lam0 <> 0 and lam1 = 0.
% Two stage estimation: OLS + maximum likelihood estimation.
% 
% This file estimate a no-arbitrage model with constant risk price. 
% Rolling estimation 120 periods from 1991:12 to 2001:11.
% Estimation window length 211; regression window 210.
% Iterated forecast.
% Forecast comparison 74 periods: 1995:11 to 2001:12. 
% Forecast comparison horizons: 1 month till 24 months ahead. 
% Some forecast before 1995:11 are not utilised, because we take these 
%   periodss as training for the program to find optimal estimate, as 
%   we input the estimate of period (t) to be the initial condition for 
%   MLE procedure of period (t+1).
%
% Variables
% m01-y10: 11 yields
% DATA: data.xls, range: 1974.2-2001.12, that corresponds to 2 - 336 row #.
% Macro panel: 171 series. First four factors (PCs) in PCL1_ih1.mat.
%
% The structure of the file is the following:
% A. Data preparation and model selection
% B. Estimation (120 periods estimation)
%    In each period:
%    1. Define data
%    2. OLS estimation of unrestricted state dynamics VAR(1)
%    3. Define initial value of parameters used for MLE on other
%       parameters.
%    4. MLE, call likelihood function llfn_obsv.m
%    5. Control result, either repeat the estimation with another initial
%         value or accept the estimate and move forward.
% C. Save estimate for later analysis.
%
%    

clear all;

%% A. ***********  DATA PREPARATION AND MODEL SPECIFICATION  ***********

% Define path and load data
xdata1=xlsread('Yields.xls'); % yield data
xdata2=xlsread('TRmacro.xls');

% Set global some parameters and intermediate result for global using or
%   the purpose of checking.
global yy Xo Xu XX Xfor H yn maturity K K2; 
global um ABne ABer aa0 bb0 AA1 BB1 VC;
global mu fai sigma lam1 crp;

% Define yield structure used in the estimation
yname =  ['m01';'m02';'m03';'m06';'m09';'y01';'y02';'y03';'y05';'y07';'y10']; 
maturity = [ 1;    2;    3;    6;    9;   12;   24;   36;   60;   84;  120];
colnumber = 2:1:12;

% +++++++++++++++++++++    Define PARAMETERS ++++++++++++++++++++++++++
yn = size(yname,1);  % number of yields
Roll=1; %Rolling estimation.
length = 335; % data length
mnlags=1; % default 12. %maximal number of lags
mnfac=2; %number of macro factors
crp = 1; % 1: constant risk price; 0: time varying risk price
START=4; 
HN = [1 3 6 9 12 18 24];% forecasting horizon
K = mnfac+1;            % number of factors 
K2 = 0;                 % number of unobservable factor(s)
lam1 = zeros(K,K);      % preset time varying risk prices to zeros.
yield= xdata1(1:length,colnumber);  


%% B. ***************************  ESTIMATION  ***************************      
ih = 1; 
H = HN(ih);     % equivalent forecast horizon in a direct forecast;
LAST=214+H;
T=length-LAST;  % left data periods for the first forecast
FT = T-(H-1);   % the forecast periods

% ooooooooooooooooooooo MAIN LOOP: estimation    oooooooooooooooooooo
k=1;
while k <= FT; 

      %:::: 1. DEFINE DATA to be used for yields and states
      yy = yield(START+(k-1)*Roll:LAST+k-1, :);  
      yy = yy'./1200; % transform to annualised return per dollar.
      un= xdata2(START+(k-1)*Roll:LAST+k-1,1)';   %Unemployment
      infl = xdata2(START+(k-1)*Roll:LAST+k-1,2)'; 
      ffr = xdata2(START+(k-1)*Roll:LAST+k-1,3)'; 
      Xo = [ones(1, size(un,2)); ffr; un; infl];
      nobs = size(Xo,2);
      nlags = mnlags;

      %:::: 2. OLS estimation of state dynamics VAR(1)
      X = Xo(:,nlags:nobs); 
      Y = Xo(2:(K+1), nlags+H:nobs);
      X = X(:, 1:nobs-nlags-H+1);
      gamav = (Y*X')/(X*X');
      omega = ((Y - gamav*X)*(Y-gamav*X)'/(nobs-H-nlags+1));
      sigma = omega;
      %Transfer VAR(1) coef. of state equation to coef. of 
      %  No-arbitrage model
      mu = gamav(:,1);
      fai = gamav(:,2:(1+K));

      %:::: 3. Set starting value for MLE.
      if k == 1;
         % At t = 1, suppose those macro factors are irrelevant.
         del1_1 = 0; 
         del1_2 = 0;
         del1_3 = 0;
         lam0_1 = 0; 
         lam0_2 = 0;
         lam0_3 = 0;
         bigthet = [  del1_1 del1_2 del1_3...
                      lam0_1 lam0_2 lam0_3]';
      else
         bigthet = thetstar; 
      end;      
      
      %:::: 4. MLE (maximize likelihood estimation)
      warning off all;
      % set options.
      if k < 2;
         options = optimset('Display','iter','MaxIter',1e+005,'MaxFunEvals',1e+7,'TolFun',1e-7,'TolX',1e-10 );
      else
         options = optimset('MaxIter',0.1*(1e+005),'MaxFunEvals',1e+6,'TolFun',1e-7,'TolX',1e-10 );
      end;
      % likelihood maximisation, call likelihood function.
      % estimated parameter values:
      thetstar = fminsearch('llfn_obsv',bigthet,options); 
      % correponding likelihood:
      fstar = llfn_obsv(thetstar)
      % collect results for each period:
      thetall(:,k,ih)= thetstar;
      fstarall(k,ih) = fstar;

      % one period ahead forecast
      mufai = [mu fai];
      Xhatk = mufai*Xfor;  Xhata(:,k,ih) = Xhatk;    
      yhatk=[AA1 BB1]*[1; Xhatk];       
      yhata(:,k,ih) = yhatk.*1200;

      k
      
      %:::: 5. Control result, either repeat the estimation with 
      %        another initial value or accept the estimate and 
      %        move forward.
      if k>1
         fdiff = fstar-fstarall(k-1,ih);
         if fdiff > 50 && ik <30 || fstar==-inf;
            k=k; ik=ik+1;
            if fstar==-inf;
               fik(:,ik)=NaN;
            else
               fik(:,ik)=fstar;
            end;
            thetik(:,:,ik)= thetstar;
            if ik==1;
               thetstar = [thetall(1:3,k-1,ih); zeros(3,1)];
            else
               thetstar = [thetall(1:3,k-1,ih); randn(3,1)*0.0001];
            end;
         elseif fdiff > 50 && ik == 30 || fstar==-inf;
            if fstar==-inf;
               fik(:,ik)=NaN;
            else
               fik(:,ik)=fstar;
            end;
            thetik(:,:,ik)= thetstar;
            [fstar, mik] = min(fik); 
            fstarall(k,ih)=fstar;
            thetall(:,k,ih)=thetik(:,:,mik);
            k=k+1; ik = 0;
         elseif fdiff<=50;
            k=k+1; ik = 0;
         end;
      elseif k == 1;
         k = k+1; ik = 0;
      end;

      fname = [cd,'\ffr_2ma_var_2s_lam01'];
      save(fname, 'thetall', 'fstarall');
end; %% end k

%oooooooooooooooo  END MAIN LOOP: rolling estimation  ooooooooooooooo

%% C. ***********     SAVE ESTIMATE FOR LATER ANALYSIS      ************
fname = [cd,'\ffr_2ma_var_2s_lam01'];
save(fname, 'thetall', 'fstarall');
