% Program for course "Topics in Econometric Models for Macroeconomics and 
% Finance" by Carlo A. Favero, IGIER, Bocconi University. (02.2007)
%
% Second stage maximum likelihood estimation: likelihood function for No-ar. 
%   model given state dynamics estimated by OLS.
%
% The structure of the file is as following:
% 1. Take initial value and define corresponding parameters;
% 2. Calculate the PDE of coefficients for yields with all maturity on 
%      the supposed measurement equations;
% 3. Define measurement equation for available yields in our data set.
% 4. Define matrices for state dynamics.
% 5. Calculate log likelihood function of our system. And penalise possible
%      constraint violations.


function llfn = llfn_A012s(bigthet)

% define variables and parameters

  global yy Xo XX Xfor H yn maturity K nlags;
  global um aa0 bb0 AA1 BB1;
  global mu fai sig del0 del1 lam1 crp;
  
  bigt = size(yy,2);
  
%% 1. Receive initial value of parameters.
lam0_1 = bigthet(1);
lam0 = [bigthet(1) zeros(1,K-1)]';

if crp == 0;
    lam1 = zeros(K,K);
    lam1(1,1) = bigthet(2);
end;


  
%% 2. Calculate the PDE of coefficients for yields with all maturity on 
%     the supposed measurement equations.
%     Factor loading matrix AA, BB for yields up to 120 periods
AA0 = zeros(1, 120);  
BB0 = zeros(K, 120);  
AA0(1) = - del0;      aa0 = -AA0;
BB0(:,1) = - del1;    bb0 = -BB0;
for i = 2 : 120;
    AA0(i) = AA0(i-1) + BB0(:,i-1)'*(mu - sig*lam0) + 0.5*BB0(:,i-1)'*sig*BB0(:,i-1) - del0;
    BB0(:,i) = (fai' - lam1'*sig')*BB0(:,i-1) - del1;
    aa0(i) = - AA0(i)./i;
    bb0(:,i) = - BB0(:,i)./i;
end;

%% 3. Define measurement equation for available yields in our data set.
AA1 = aa0(maturity')';
BB1 = bb0(:, maturity')';
AB = [AA1(2:yn,:) BB1(2:yn,:)];
  
%% 4. Define matrices for the state dynamics.
XX = Xo;  
Xfor = XX(:,size(XX,2));  %last observation of the interval

%% 5. Calculate log likelihood function of our system.
% Term four: -.5*(T-1)*log(sum(i=1:N-K2, sigma(i)^2);
% sigma(i)^2 is the variance of yield (er) prediction error 

  um = yy(2:yn,nlags:bigt) - (AB * XX);     %it picks from the second maturity!
  VC = (1/(bigt-nlags-1))*um*um'; % variance-covariance matrix of yer
  V = diag(diag(VC));
  t4 = -.5*(bigt-nlags-1)*log(sum(diag(V))); 
  
% Term five: -.5*(sum t=2:T (sum i=1:yn-K2 um(t,i)^2/sigma(i)^2))
  % um = um(:,H+1:bigt);
  umsq = um.^2;
  umsqw = V\umsq;  % weighted square residual;
  sum_umsqw = sum(sum(umsqw,2));
  t5 = -.5 * sum_umsqw;
  
  llfn = -(t4 + t5);
  

  % penalize constraint violations 
  if abs(imag(llfn)) > 0
     llfn = real(llfn) + 1e8;
  end
  