% Program for course "Topics in Econometric Models for Macroeconomics and
% Finance" by Carlo A. Favero, IGIER, Bocconi University. (02.2007)
%
% Maximum likelihood estimation: likelihood function for A0(1) with
%   constant or time-varying risk price.
%
% The structure of the file is as following:
% 1. Take initial value and define corresponding parameters;
% 2. Calculate the PDE of coefficients for yields with all maturity on
%      the supposed measurement equations;
% 3. Define measurement equation for available yields in our data set.
% 4. Divide the measurement equation into two groups: ne, er.
% 5. Extract latent variables from the measurement equation of ne yields.
% 6. Define matices for the state dynamics.
% 7. Calculate log likelihood function of our system. And panelise possible
%      constraint violations.


function llfn = llfn_A01CS(bigthet)

% define variables and parameters

global yy Xu Xu XX Xfor yn maturity K ne er;
global um ABne ABer aa0 bb0 AA1 BB1;
global mu fai sig del0 del1 lam1 crp;

bigt = length(yy);

%% 1. Receive initial value of parameters.

fai = bigthet(1);

sig = bigthet(2);

lam0 = bigthet(3);

if crp == 0;
    lam1 = bigthet(4);
end;

%% 2. Calculate the PDE of coefficients for yields with all maturity on
%     the supposed measurement equations;

% Factor loading matrix AA, BB for yields up to 120 months
AA0 = zeros(1, 120);
BB0 = zeros(K, 120);
AA0(1) = - del0;      aa0 = -AA0;
BB0(:,1) = - del1;    bb0 = -BB0;
for i = 2 : 120;
    AA0(i) = AA0(i-1) + BB0(:,i-1)'*(mu - sig*lam0) + 0.5*BB0(:,i-1)'*sig*BB0(:,i-1) - del0;
    BB0(:,i) = (fai' - lam1'*sig')*BB0(:,i-1) - del1;
    aa0(i) = - AA0(i)./i;
    bb0(:,i) = - BB0(:,i)./i;
end;

%% 3. Define measurement equation for available yields in our data set.

AA1 = aa0(maturity')';
BB1 = bb0(:, maturity')';

%% 4. Divide the measurement equation into two groups: ne, er.
%     Devide the yields into yne(measured without error) and
%     yer(measured with error) and reshape the AA, BB coefficients
%     for these yields respectively.
yne = yy(ne, :);
yer = yy(er, :);

AAne = AA1(ne,:);
BBne = BB1(ne,:);
ABne = [AAne BBne];

AAer = AA1(er,:);
BBer = BB1(er,:);
ABer = [AAer BBer];

%% 5. Extract latent variables from the measurement equation of ne yields.
Xu = BBne\eye(size(BBne,1))*(yne - AAne * ones(1, bigt));  

% there are no observable factors in the equation

%% 6. Define matrices for the state dynamics.
X = Xu; % LHS
XX = [ones(1, bigt); Xu]; % RHS
Xfor = XX(:,bigt);  %the last period state variable for forecast


%% 7. Calculate log likelihood function of our system.
% Term one: -(T-1)log|det(J)|, where det(J) = det(BBne)
detJ = det(BBne);
t1 = -(bigt - 1 ) * log(abs(detJ));

% Term two: -(T-1)/2 * log(det(sig))
det_sig = det(sig);
t2 = -.5*(bigt - 1)*log(det_sig);

% Term three: -.5* sum[(X_t - mu - fai*X_t-h)'*inv(sig)*(X_t - mu -
% fai*X_t-h)]
mufai = [mu fai];
ita = X(:, 2:bigt) - mufai * XX(:,1:(bigt-1));
inv_sig = sig\eye(K);
sqrX = ita'*inv_sig*ita;  % element of factor error decomposition
t3 = -.5 * sum(diag(sqrX));

% Term four: -.5*(T-1)*log(sum(i=1:N-K2, sigma(i)^2);
% sigma(i)^2 is the variance of yield (er) prediction error
um = yer - ABer * XX;
VC = (1/(bigt-1))*um*um'; % variance-covariance matrix of yer
V = diag(diag(VC));
t4 = -.5*(bigt-1)*log(sum(diag(V)));

% Term five: -.5*(sum t=2:T (sum i=1:yn-K2 um(t,i)^2/sigma(i)^2))
% um = um(:,H+1:bigt);
umsq = um.^2;
umsqw = V\umsq;  % weighted square residual;
sum_umsqw = sum(sum(umsqw,2));
t5 = -.5 * sum_umsqw;

llfn = -(t1 + t2 + t3 + t4 + t5);   %change the sign


% penalize constraint violations
if sig<0;
    llfn = real(llfn) + 1e8;
end
if abs(imag(llfn)) > 0
    llfn = real(llfn) + 1e8;
end
