% Program for course "Topics in Econometric Models for Macroeconomics and
% Finance" by Carlo A. Favero, IGIER, Bocconi University. (02.2007)
%
% Maximum likelihood estimation: likelihood function for A0(1) with
%   constant or time-varying risk price, by Kalman Filter.
%
% The structure of the file is as following:
% 1. Take initial value and define corresponding parameters;
% 2. Calculate the PDE of coefficients for yields with all maturity on
%      the supposed measurement equations;
% 3. Define measurement equation for available yields in our data set.
% 4. Calculate log likelihood function of our system. And panelise possible
%      constraint violations.

function llfn = llfn_A01KF(bigthet)

% define variables and parameters

global yy XX Xfor yn maturity K k X1 Xu;
global um aa0 bb0 AA1 BB1;
global mu fai crp del0 del1 lam1;
global sigmat_lag bigB ft_lag Xiniset;

bigt = length(yy);


%% 1. Receive initial value of parameters.

fai = bigthet(1);

omega = bigthet(2);

sig = bigthet(3);

lam0 = bigthet(4);

if crp == 0;
    lam1 = bigthet(5);
end;

%% 2. Calculate the PDE of coefficients for yields with all maturity on
%     the supposed measurement equations;

% Factor loading matrix AA, BB for yields up to 120 periods
AA0 = zeros(1, 120);
BB0 = zeros(K, 120);
AA0(1) = - del0;      aa0 = -AA0;
BB0(:,1) = - del1;    bb0 = -BB0;
for i = 2 : 120;
    AA0(i) = AA0(i-1) + BB0(:,i-1)'*(mu - omega*lam0) + 0.5*BB0(:,i-1)'*omega*BB0(:,i-1) - del0;
    BB0(:,i) = (fai' - lam1'*omega')*BB0(:,i-1) - del1;
    aa0(i) = - AA0(i)./i;
    bb0(:,i) = - BB0(:,i)./i;
end;

%% 3. Define measurement equation for available yields in our data set.

AA1 = aa0(maturity')';
BB1 = bb0(:, maturity')';


%% 4. Calculate log likelihood function of our system. And panelise possible
%      constraint violations.
% initial log likelihood function with Kalman Filter
llfn = -0.5*yn*(bigt-1)*log(2*pi);

DD = eye(K);

bigB = [AA1 BB1];

if k == 1;
    Xini = zeros(K,1);
else
    Xini = Xiniset;
end;

% 1) Initial values of X and sigma, the unconditional cov matrix of X

Xt_t = Xini;
inter = eye(K^2) - kron(fai,fai');
DomegaD = DD*omega*DD';
vecsigma = inter\eye(K^2) * DomegaD(:);
sigmat_t = reshape(vecsigma, K,  K);

for t = 1:bigt;
    Xt_lag = fai*Xt_t;
    sigmat_lag = fai*sigmat_t*fai' + DD*omega*DD';
    itat_lag = yy(:,t) -  AA1 - BB1*Xt_lag;
    ft_lag = BB1*sigmat_lag*BB1'+sig^2*eye(yn);
    
    %% likelihood function:
    if t>1;  %% Count from 2nd periods forward.
        llfn = llfn - 0.5*log(det(ft_lag)) - 0.5*itat_lag'*(ft_lag\eye(yn))*itat_lag;
    end;
    
    %% updating:
    Kalg = sigmat_lag*BB1'*(ft_lag\eye(yn));  %% Kalman gain
    Xt_t = Xt_lag + Kalg*itat_lag;
    if t == 1;
        X1 = Xt_t;
    end;
    sigmat_t = sigmat_lag - Kalg*BB1*sigmat_lag;
    %Xu(t,:)=Xt_t;
end;

Xfor = [1; Xt_t(1:K,:)];

llfn = -llfn;


if omega < 0;
    llfn = real(llfn) + 1e8;
end
if abs(imag(llfn)) > 0
    llfn = real(llfn) + 1e8;
end

