% Program for paper "Term Structure Forecast: No-arbitrage Restrictions vs.
%   Large Information set" (2007) by Carlo A. Favero, Linlin Niu and Luca 
%   Sala, IGIER, Bocconi University.
%
% Maximum likelihood estimation: likelihood function for three latent yield
% factors with one or two observable factores with constant or time-varying
% risk price.
%
% The structure of the file is as following:
% 1. Take initial value and define corresponding parameters;
% 2. Calculate the PDE of coefficients for yields with all maturity on 
%      the supposed measurement equations;
% 3. Define measurement equation for available yields in our data set.
% 4. Divide the measurement equation into two groups: ne, er.
% 5. Extract latent variables from the measurement equation of ne yields.
% 6. Define matices for the state dynamics.
% 7. Calculate log likelihood function of our system. And panelise possible
%      constraint violations.

function [llfn ll] = llfn_mix_sd(bigthet)

% define variables and parameters

  global yy Xo Xu XX Xfor yn maturity yne yer meanX K K2;
  global inv_V um ABne ABer aa0 bb0 AA1 BB1 VC ne er;
  global mu fai sig lam1 crp;
  
  bigt = size(yy,2);

%% 1. Receive initial value of parameters.

if K == 3;
fai = [bigthet(1) 0          0         
       bigthet(2) bigthet(3) 0         
       bigthet(4) bigthet(5) bigthet(6)];
elseif K == 4;
fai = [bigthet(1) 0          0          0
       bigthet(2) bigthet(3) 0          0
       bigthet(4) bigthet(5) bigthet(6) 0
       bigthet(7) bigthet(8) bigthet(9) bigthet(10)];
elseif K == 5;
fai = [bigthet(1)  0           0           0           0
       bigthet(2)  bigthet(3)  0           0           0 
       bigthet(4)  bigthet(5)  bigthet(6)  0           0
       bigthet(7)  bigthet(8)  bigthet(9)  bigthet(10) 0
       bigthet(11) bigthet(12) bigthet(13) bigthet(14) bigthet(15)];
elseif K == 7;
fai = [bigthet(1)  0           0           0           0           0           0
       bigthet(2)  bigthet(3)  0           0           0           0           0
       bigthet(4)  bigthet(5)  bigthet(6)  0           0           0           0
       bigthet(7)  bigthet(8)  bigthet(9)  bigthet(10) 0           0           0
       bigthet(11) bigthet(12) bigthet(13) bigthet(14) bigthet(15) 0           0
       bigthet(16) bigthet(17) bigthet(18) bigthet(19) bigthet(20) bigthet(21) 0
       bigthet(22) bigthet(23) bigthet(24) bigthet(25) bigthet(26) bigthet(27) bigthet(28)];    
end;
mu = inv(eye(K)-fai)*meanX;

%sig = diag(bigthet(0.5*(K+1)*K + 1 : 0.5*(K+1)*K + K));

del1 = [bigthet(0.5*(K+1)*K +K+1-K:0.5*(K+1)*K +2*K -K2-K);  ones(K2,1)];

del0 = mean(yy(1,:))-mean(Xo')*del1(1:K-K2);

lam0 = bigthet(0.5*(K+1)*K +2*K-K2+1-K : 0.5*(K+1)*K +3*K-K2-K);

if crp == 0;
    lam1 = diag(bigthet(0.5*(K+1)*K + 3*K-K2+1: 0.5*(K+1)*K + 4*K-K2));
end;
 

%% 2. Calculate the PDE of coefficients for yields with all maturity on 
%     the supposed measurement equations;

% Factor loading matrix AA, BB for yields up to 120 periods
AA0 = zeros(1, 120);  
BB0 = zeros(K, 120);  
AA0(1) = - del0;      aa0 = -AA0;
BB0(:,1) = - del1;    bb0 = -BB0;
for i = 2 : 120;
    AA0(i) = AA0(i-1) + BB0(:,i-1)'*(mu - sig*lam0) + 0.5*BB0(:,i-1)'*sig*BB0(:,i-1) - del0;
    BB0(:,i) = (fai' - lam1'*sig')*BB0(:,i-1) - del1;
    aa0(i) = - AA0(i)./i;
    bb0(:,i) = - BB0(:,i)./i;
end;


%% 3. Define measurement equation for available yields in our data set.

AA1 = aa0(maturity')';
BB1 = bb0(:, maturity')';


%% 4. Divide the measurement equation into two groups: ne, er.
%     Devide the yields into yne(measured without error) and 
%     yer(measured with error) and reshape the AA, BB coefficients
%     for these yields respectively.
yne = yy(ne, :);
yer = yy(er, :);  

AAne = AA1(ne,:);
BBne = BB1(ne,:);
ABne = [AAne BBne];

BBne_o=BBne(:, 1:(K-K2));
BBne_u=BBne(:, (K-K2+1):K);
ABne_o = [AAne BBne_o];  % write the coefficient in one vector
  
AAer = AA1(er, :);
BBer = BB1(er, :); 
ABer = [AAer BBer];
  

%% 5. Extract latent variables from the measurement equation of ne yields.
XXo = [ones(1, bigt); Xo];   % add one as the first variable
Xu = BBne_u\eye(size(BBne_u,1))*(yne - ABne_o * XXo);


%% 6. Define matices for the state dynamics.
X = [Xo; Xu]; % LHS
XX = [XXo; Xu]; % RHS
Xfor = XX(:,bigt);  %the last period state variable for forecast
%Xu = Xu(:, H+1:bigt); % Xu corresponding to yy(:, H+1:bigt);

%% 7. Calculate log likelihood function of our system.
% Term one: -(T-1)log|det(J)|, where det(J) = det(BBne)
  detJ = det(BBne_u);
%   t1 = -(bigt - 2) * log(abs(detJ));
  t11 = -log(abs(detJ));
  t12 = -(bigt - 2) * log(abs(detJ));
% Term two: -(T-1)/2 * log(det(sig))
%   t2 = -.5*(bigt-2)*log(det_sig);
  t21 = -.5*log(det(sig));
  t22 = -.5*(bigt-2)*log(det(sig));
  
% Term three: -.5* sum[(X_t - mu - fai*X_t-h)'*inv(sig)*(X_t - mu -
% fai*X_t-h)]
  mufai = [mu fai];
  ita = X(:, 2:bigt) - mufai * XX(:,1:(bigt-1));
  inv_sig = sig\eye(K);
  sqrX = ita'*inv_sig*ita;  % element of factor error decomposition 
%   t3 = -.5 * sum(diag(sqrX));
  t31 = -.5 * diag(sqrX);
  t32 = -.5 * sum(diag(sqrX));
% Term four: -.5*(T-1)*log(sum(i=1:N-K2, sigma(i)^2);
% sigma(i)^2 is the variance of yield (er) prediction error 
  um = yer(:, 2:bigt) - ABer * XX(:, 2:bigt);
  VC = (1/(bigt-2))*um*um'; % variance-covariance matrix of yer
  V = diag(diag(VC));
%   t4 = -.5*(bigt-2)*log(sum(diag(V)));
  t41 = -.5*log(det(V));
  t42 = -.5*(bigt-1)*log(det(V));
% Term five: -.5*(sum t=2:T (sum i=1:yn-K2 um(t,i)^2/sigma(i)^2))
  % um = um(:,H+1:bigt);
  umsq = um.^2;
  umsqw = V\umsq;  % weighted square residual;
  sum_umsqw = sum(sum(umsqw,2));
%   t5 = -.5 * sum_umsqw;
%   
%   llfn = -(t1 + t2 + t3 + t4 + t5);
  t51 = -.5 * sum(umsqw,1);
  t51 = t51';
  t52 = -.5 * sum_umsqw;
  
  ll = (t11 + t21 + t31 + t41 + t51)+1/2 *log(2*pi);
  llfn = (t12 + t22 + t32 + t42 + t52)+(bigt-1)/2 *log(2*pi);  

  % penalize constraint violations 
  if sig<=0;
    llfn = real(llfn) + 1e8;
  end
  if abs(imag(llfn)) > 0
    llfn = real(llfn) + 1e8;
  end
  

  