% Program for paper "Term Structure Forecast: No-arbitrage Restrictions vs.
%   Large Information set" (2007) by Carlo A. Favero, Linlin Niu and Luca 
%   Sala, IGIER, Bocconi University.
%
% Second stage maximum likelihood estimation: likelihood function for No-ar. 
%   constant risk price model given state dynamics estimated by OLS.
%
% The structure of the file is as following:
% 1. Take initial value and define corresponding parameters;
% 2. Calculate the PDE of coefficients for yields with all maturity on 
%      the supposed measurement equations;
% 3. Define measurement equation for available yields in our data set.
% 4. Define matrices for state dynamics.
% 5. Calculate log likelihood function of our system. And panelise possible
%      constraint violations.


function llfn = llfn_obsv(bigthet)

% define variables and parameters

  global yy Xo Xu XX Xfor H yn maturity K K2;
  global um ABne ABer aa0 bb0 AA1 BB1 VC;
  global mu fai sigma lam1 crp;
  
  bigt = length(yy);
  
%% 1. Receive initial value of parameters.
del1 = bigthet(1:K)./(1e+4);

del0 = mean(yy(1,:))-del1'*mean(Xo(2:K+1,:)')';

lam0 = bigthet(K+1:2*K);

if crp == 0;
    lam1 = diag(bigthet(2*K+1:3*K));
end;

sig = sigma;

  
%% 2. Calculate the PDE of coefficients for yields with all maturity on 
%     the supposed measurement equations.
%     Factor loading matrix AA, BB for yields up to 120 periods
AA0 = zeros(1, 120);  
BB0 = zeros(K, 120);  
AA0(1) = - del0;      aa0 = -AA0;
BB0(:,1) = - del1;    bb0 = -BB0;
for i = 2 : 120;
    AA0(i) = AA0(i-1) + BB0(:,i-1)'*(mu - sig*lam0) + 0.5*BB0(:,i-1)'*sig*BB0(:,i-1) - del0;
    BB0(:,i) = (fai' - lam1'*sig')*BB0(:,i-1) - del1;
    aa0(i) = - AA0(i)./i;
    bb0(:,i) = - BB0(:,i)./i;
end;

%% 3. Define measurement equation for available yields in our data set.
AA1 = aa0(maturity')';
BB1 = bb0(:, maturity')';
AB = [AA1 BB1];
  
%% 4. Define matices for the state dynamics.
XX = Xo;  
Xfor = XX(:,bigt);

%% 7. Calculate log likelihood function of our system.
% Term four: -.5*(T-1)*log(sum(i=1:N-K2, sigma(i)^2);
% sigma(i)^2 is the variance of yield (er) prediction error 
  um = yy - (AB * XX);
  VC = (1/(bigt-H-1))*um*um'; % variance-covariance matrix of yer
  V = diag(diag(VC));
  %inv_V = V\eye(yn);
  t4 = -.5*(bigt-H)*log(sum(diag(V))); 
  
% Term five: -.5*(sum t=2:T (sum i=1:yn-K2 um(t,i)^2/sigma(i)^2))
  % um = um(:,H+1:bigt);
  umsq = um.^2;
  umsqw = V\umsq;  % weighted square residual;
  sum_umsqw = sum(sum(umsqw,2));
  t5 = -.5 * sum_umsqw;
  
  llfn = -(t4 + t5);
  

  % penalize constraint violations 
  if abs(imag(llfn)) > 0
     llfn = real(llfn) + 1e8;
  end
  